//
// Created by zx on 23-2-22.
//

#include "mqttmsg.h"


MqttMsg::MqttMsg()
{
  data_= nullptr;
  length_=0;
}
MqttMsg::MqttMsg(char* data,const int length)
{
  data_=(char*)malloc(length);
  memcpy(data_,data,length);
  length_=length;
}
MqttMsg::MqttMsg(const MqttMsg& msg)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_)
    free(data_);
  data_=(char*)malloc(msg.length_);
  memcpy(data_,msg.data_,msg.length_);
  length_=msg.length_;
}
MqttMsg& MqttMsg::operator=(const MqttMsg& msg)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_)
    free(data_);
  data_=(char*)malloc(msg.length_);
  memcpy(data_,msg.data_,msg.length_);
  length_=msg.length_;
  return *this;
}

char* MqttMsg::data()const{
  return data_;
}
int MqttMsg::length()const{
  return length_;
}

void MqttMsg::fromPclPointCloud(pcl::PointCloud<pcl::PointXYZ>::Ptr cloud)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_)
    free(data_);
  if(cloud->size()==0)
  {
    data_= nullptr;
    length_=0;
  }
  length_=4+cloud->size()*3*sizeof(float);
  data_=(char*)malloc(length_);
  int type=ePointCloudXYZ;
  memcpy(data_,&type,4);

  int point_bytes=3*sizeof(float);
  for(int i=0;i<cloud->size();++i)
  {
    float d[3]={cloud->points[i].x,cloud->points[i].y,cloud->points[i].z};
    memcpy(data_+4+i*(point_bytes),d,point_bytes);
  }
}
MqttMsg::~MqttMsg(){
  if(data_!= nullptr)
  {
    free(data_);
    data_ = nullptr;
  }
  length_=0;
}


bool MqttMsg::toPclCloud(pcl::PointCloud<pcl::PointXYZ>::Ptr cloud)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_== nullptr || length_<4)
    return false;

  int type=-1;
  memcpy(&type,data_,sizeof(int));
  printf("type:%d\n",type);
  if(type!=ePointCloudXYZ) return false;

  int point_bytes=3*sizeof(float);
  if((length_-4)%point_bytes!=0) return false;

  cloud->clear();
  int i=0;
  while(i<length_-4)
  {
    float d[3]={0};
    memcpy(d,data_+i+4,point_bytes);
    pcl::PointXYZ point;
    point.x=d[0];
    point.y=d[1];
    point.z=d[2];
    cloud->push_back(point);
    i+=point_bytes;

  }

  return true;
}
bool MqttMsg::toPclCloud(pcl::PointCloud<pcl::PointXYZI>::Ptr cloud)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_== nullptr || length_<4)
    return false;
  int type=-1;
  memcpy(&type,data_,sizeof(int));
  return true;
}
bool MqttMsg::toPclCloud(pcl::PointCloud<pcl::PointXYZRGB>::Ptr cloud)
{
  std::lock_guard<std::mutex> lock(mutex_);
  if(data_== nullptr || length_<4)
    return false;
  int type=-1;
  memcpy(&type,data_,sizeof(int));
  return true;
}